import importlib.util, pkg_resources


class PackageUtils(object):
    @staticmethod
    def getPackageLocation(package):
        """
        Attempts to retrieve the filesystem location for the specified Python package
        """
        return pkg_resources.get_distribution(package).location

    @staticmethod
    def importFile(moduleName, filePath):
        """
        Directly imports a Python module from a source file
        """
        spec = importlib.util.spec_from_file_location(moduleName, filePath)
        module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(module)
        return module



from packaging import version
import os, platform


class DarwinUtils(object):
    @staticmethod
    def minimumRequiredVersion():
        """
        Returns the minimum required version of macOS, which is 10.10.3 Yosemite

        (10.10.3 is the minimum required version for Docker for Mac, as per:
        <https://store.docker.com/editions/community/docker-ce-desktop-mac>)
        """
        return "10.10.3"

    @staticmethod
    def systemString():
        """
        Generates a human-readable version string for the macOS host system
        """
        return "macOS {} (Kernel Version {})".format(
            platform.mac_ver()[0], platform.uname().release
        )

    @staticmethod
    def getMacOsVersion():
        """
        Returns the version number for the macOS host system
        """
        return platform.mac_ver()[0]

    @staticmethod
    def isSupportedMacOsVersion():
        """
        Verifies that the macOS host system meets our minimum version requirements
        """
        return version.parse(DarwinUtils.getMacOsVersion()) >= version.parse(
            DarwinUtils.minimumRequiredVersion()
        )



from .infrastructure import DockerUtils, GlobalConfiguration, Logger
from container_utils import ContainerUtils, ImageUtils
import docker, os, platform, sys


def test():

    # Create our logger to generate coloured output on stderr
    logger = Logger(prefix="[{} test] ".format(sys.argv[0]))

    # Create our Docker API client
    client = docker.from_env()

    # Check that an image tag has been specified
    if len(sys.argv) > 1 and sys.argv[1].strip("-") not in ["h", "help"]:

        # Verify that the specified container image exists
        tag = sys.argv[1]
        image = GlobalConfiguration.resolveTag(
            "ue4-full:{}".format(tag) if ":" not in tag else tag
        )
        if DockerUtils.exists(image) == False:
            logger.error(
                'Error: the specified container image "{}" does not exist.'.format(
                    image
                )
            )
            sys.exit(1)

        # Use process isolation mode when testing Windows containers, since running Hyper-V containers don't currently support manipulating the filesystem
        platform = ImageUtils.image_platform(client, image)
        isolation = "process" if platform == "windows" else None

        # Start a container to run our tests in, automatically stopping and removing the container when we finish
        logger.action(
            'Starting a container using the "{}" image...'.format(image), False
        )
        container = ContainerUtils.start_for_exec(client, image, isolation=isolation)
        with ContainerUtils.automatically_stop(container):

            # Create the workspace directory in the container
            workspaceDir = ContainerUtils.workspace_dir(container)
            ContainerUtils.exec(
                container,
                ContainerUtils.shell_prefix(container) + ["mkdir " + workspaceDir],
            )

            # Copy our test scripts into the container
            testDir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "tests")
            ContainerUtils.copy_from_host(container, testDir, workspaceDir)

            # Create a harness to invoke individual tests
            containerPath = ContainerUtils.path(container)
            pythonCommand = (
                "python"
                if ContainerUtils.container_platform(container) == "windows"
                else "python3"
            )

            def runTest(script):
                logger.action('Running test "{}"...'.format(script), False)
                try:
                    ContainerUtils.exec(
                        container,
                        [pythonCommand, containerPath.join(workspaceDir, script)],
                        workdir=workspaceDir,
                    )
                    logger.action('Passed test "{}"'.format(script), False)
                except RuntimeError as e:
                    logger.error('Error: test "{}" failed!'.format(script))
                    raise e from None

            # Run each of our tests in turn
            runTest("build-and-package.py")
            runTest("consume-external-deps.py")

            # If we've reached this point then all of the tests passed
            logger.action("All tests passed.", False)

    else:

        # Print usage syntax
        print("Usage: {} test TAG".format(sys.argv[0]))
        print("Runs tests to verify the correctness of built container images\n")
        print("TAG should specify the tag of the ue4-full image to test.")



from os.path import abspath, dirname, join
from setuptools import setup

# Read the README markdown data from README.md
with open(abspath(join(dirname(__file__), "README.md")), "rb") as readmeFile:
    __readme__ = readmeFile.read().decode("utf-8")

# Read the version number from version.py
with open(abspath(join(dirname(__file__), "ue4docker", "version.py"))) as versionFile:
    __version__ = (
        versionFile.read().strip().replace("__version__ = ", "").replace('"', "")
    )

setup(
    name="ue4-docker",
    version=__version__,
    description="Windows and Linux containers for Unreal Engine 4",
    long_description=__readme__,
    long_description_content_type="text/markdown",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Software Development :: Build Tools",
        "Environment :: Console",
    ],
    keywords="epic unreal engine docker",
    url="http://github.com/adamrehn/ue4-docker",
    author="Adam Rehn",
    author_email="adam@adamrehn.com",
    license="MIT",
    packages=[
        "ue4docker",
        "ue4docker.diagnostics",
        "ue4docker.exports",
        "ue4docker.infrastructure",
    ],
    zip_safe=False,
    python_requires=">=3.6",
    install_requires=[
        "colorama",
        "container-utils",
        "docker>=3.0.0",
        "humanfriendly",
        "Jinja2>=2.11.3",
        "packaging>=19.1",
        "psutil",
        "requests",
        "semver>=2.7.9,<3.0.0",
        # We want newer setuptools to avoid shim path bug on newer Pythons.
        # See #231 and https://github.com/pypa/setuptools/issues/3001
        # Unfortunately, that version of setuptools doesn't have Python 3.6 support
        "setuptools>=60.4.0;python_version>='3.7.0'",
        "termcolor",
        "twine>=1.11.0",
        "wheel>=0.31.0",
    ],
    package_data={
        "ue4docker": [
            "dockerfiles/*/*/.dockerignore",
            "dockerfiles/diagnostics/*/*/*",
            "dockerfiles/*/*/*",
            "tests/*.py",
        ]
    },
    entry_points={"console_scripts": ["ue4-docker=ue4docker:main"]},
)
